@@ -565,6 +565,10 @@ get_workspace_rect (WnckPager    *pager,
 
       col = space / spaces_per_row;
       row = space % spaces_per_row;
+
+      if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
+        col = pager->priv->n_rows - col - 1;
+
       rect->x = (rect->width + 1) * col; 
       rect->y = (rect->height + 1) * row;
       
@@ -581,6 +585,10 @@ get_workspace_rect (WnckPager    *pager,
       
       col = space % spaces_per_row;
       row = space / spaces_per_row;
+
+      if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
+        col = spaces_per_row - col - 1;
+
       rect->x = (rect->width + 1) * col; 
       rect->y = (rect->height + 1) * row;
 
@@ -930,7 +938,7 @@ wnck_pager_draw_workspace (WnckPager    *pager,
                                      GDK_RGB_DITHER_MAX,
                                      0, 0);
     }
-  else
+  else if (active_space)
     {
       int vx, vy, vw, vh; /* viewport */
       double width_ratio, height_ratio;
@@ -1323,12 +1331,18 @@ wnck_pager_drag_data_received (GtkWidget          *widget,
 
   if ((selection_data->length != sizeof (gulong)) ||
       (selection_data->format != 8))
-    gtk_drag_finish (context, FALSE, FALSE, time);
+    {
+      gtk_drag_finish (context, FALSE, FALSE, time);
+      return;
+    }
   
   i = workspace_at_point (pager, x, y, NULL, NULL);
   space = wnck_screen_get_workspace (pager->priv->screen, i);
   if (!space)
-    gtk_drag_finish (context, FALSE, FALSE, time);
+    {
+      gtk_drag_finish (context, FALSE, FALSE, time);
+      return;
+    }
   
   xid = *((gulong *)selection_data->data);
 	      
@@ -1358,7 +1372,7 @@ wnck_pager_drag_data_get (GtkWidget        *widget,
   WnckPager *pager = WNCK_PAGER (widget);
   gulong xid;
 
-  if (pager->priv->drag_window == NULL) 
+  if (pager->priv->drag_window == NULL)
     return;
 
   xid = wnck_window_get_xid (pager->priv->drag_window);
@@ -1410,6 +1424,7 @@ wnck_update_drag_icon (WnckWindow     *window,
 		       GdkDragContext *context)
 {
   gint org_w, org_h, dnd_w, dnd_h;
+  WnckWorkspace *workspace;
   GdkRectangle rect;
   GdkPixmap *pixmap;
   GtkWidget *widget;
@@ -1418,27 +1433,36 @@ wnck_update_drag_icon (WnckWindow     *window,
   if (!gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (widget),
 					  GTK_ICON_SIZE_DND, &dnd_w, &dnd_h))
     dnd_w = dnd_h = 32;
-    
   /* windows are huge, so let's make this huge */
-  dnd_w *= 2;
-  dnd_h *= 2;
-    
+  dnd_w *= 3;
+ 
+  workspace = wnck_window_get_workspace (window);
+  if (workspace == NULL)
+    workspace = wnck_screen_get_active_workspace (wnck_window_get_screen (window));
+  if (workspace == NULL)
+    return;
+
   wnck_window_get_geometry (window, NULL, NULL, &org_w, &org_h);
+
   rect.x = rect.y = 0;
-  rect.width = sqrt (dnd_w * dnd_h * org_w / org_h);
+  rect.width = 0.5 + ((double) (dnd_w * org_w) / (double) wnck_workspace_get_width (workspace));
   rect.width = MIN (org_w, rect.width);
-  rect.width = MAX (rect.width, 2 + DEFAULT_ICON_WIDTH);
-  rect.height = sqrt (dnd_w * dnd_h * org_h / org_w);
-  rect.height = MIN (org_h, rect.height);
-  rect.height = MAX (rect.height, 2 + DEFAULT_ICON_HEIGHT);
+  rect.height = 0.5 + ((double) (rect.width * org_h) / (double) org_w);
+
+  /* we need at least three pixels to draw the smallest window */
+  rect.width = MAX (rect.width, 3);
+  rect.height = MAX (rect.height, 3);
+
   pixmap = gdk_pixmap_new (GTK_WIDGET (widget)->window,
                            rect.width, rect.height, -1);
   draw_window (GDK_DRAWABLE (pixmap), widget, window,
 	       &rect, GTK_STATE_NORMAL, FALSE);  
+
   gtk_drag_set_icon_pixmap (context, 
                             gdk_drawable_get_colormap (GDK_DRAWABLE (pixmap)),
 			    pixmap, NULL,
-			    rect.width / 2, 0);
+			    rect.width / 4, 0);
+
   g_object_unref (pixmap);
 }
 
@@ -1448,11 +1472,6 @@ wnck_drag_window_destroyed (gpointer  contextp,
 {
   wnck_drag_clean_up ((WnckWindow *) window, GDK_DRAG_CONTEXT (contextp),
                       FALSE, TRUE);
-  /* FIXME: I'd like to have a "window is destroyed" indication, but no idea
-   * how best to do it.
-   * I didn't like setting GTK_STOCK_MISSING_IMAGE for example, because
-   * that indicates a missing image, not a missing window */
-  gtk_drag_set_icon_default (contextp);
 }
 
 /* CAREFUL: This function is a bit brittle, because the pointers given may be
@@ -1704,6 +1723,7 @@ wnck_pager_set_n_rows (WnckPager *pager,
 		       int        n_rows)
 {
   g_return_if_fail (WNCK_IS_PAGER (pager));
+  g_return_if_fail (n_rows > 0);
 
   if (pager->priv->n_rows == n_rows)
     return;
