@@ -67,7 +67,8 @@ typedef enum
   WNCK_WINDOW_ACTION_MAXIMIZE                = 1 << 14,
   WNCK_WINDOW_ACTION_UNMAXIMIZE              = 1 << 15,
   WNCK_WINDOW_ACTION_FULLSCREEN              = 1 << 16,
-  WNCK_WINDOW_ACTION_ABOVE                   = 1 << 17
+  WNCK_WINDOW_ACTION_ABOVE                   = 1 << 17,
+  WNCK_WINDOW_ACTION_BELOW                   = 1 << 18
 } WnckWindowActions;
 
 typedef enum
@@ -94,7 +95,8 @@ typedef enum
   WNCK_WINDOW_GRAVITY_EAST      = 6,
   WNCK_WINDOW_GRAVITY_SOUTHWEST = 7,
   WNCK_WINDOW_GRAVITY_SOUTH     = 8,
-  WNCK_WINDOW_GRAVITY_SOUTHEAST = 9
+  WNCK_WINDOW_GRAVITY_SOUTHEAST = 9,
+  WNCK_WINDOW_GRAVITY_STATIC    = 10
 } WnckWindowGravity;
 
 typedef enum
@@ -102,7 +104,7 @@ typedef enum
   WNCK_WINDOW_CHANGE_X      = 1 << 0,
   WNCK_WINDOW_CHANGE_Y      = 1 << 1,
   WNCK_WINDOW_CHANGE_WIDTH  = 1 << 2,
-  WNCK_WINDOW_CHANGE_HEIGHT = 1 << 3,
+  WNCK_WINDOW_CHANGE_HEIGHT = 1 << 3
 } WnckWindowMoveResizeMask;
 
 #define WNCK_TYPE_WINDOW              (wnck_window_get_type ())
