@@ -22,7 +22,8 @@
  */
 
 #include <config.h>
-#include <libintl.h>
+
+#include <glib/gi18n-lib.h>
 #include "workspace.h"
 #include "xutils.h"
 #include "private.h"
@@ -62,7 +63,7 @@ wnck_workspace_get_type (void)
   
   if (!object_type)
     {
-      static const GTypeInfo object_info =
+      const GTypeInfo object_info =
       {
         sizeof (WnckWorkspaceClass),
         (GBaseInitFunc) NULL,
